!******************************************************************************!
!                                                                              !
!    class_vertices.f90                                                        !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   class_vertices.f90                             |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 03/2019                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2019-03-21 23:40             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module class_vertices
  use fill_couplings
  use class_particles
  use class_ctparameters
  use fill_ctparameters, only: init_CTparameters_mdl
  use particle_keys, only: createParticleConfig
  use dictionary, only: dict,dict_get_value,dict_init
  use constants_mdl, only: Nc
  implicit none
  save

  interface get_vertex_mdl
    module procedure get_vertex3_mdl, get_vertex4_mdl, get_vertexN_mdl
  end interface

  type Branch
    integer :: lorentz_id
    integer :: perm
    integer, allocatable :: couplings(:)
    integer, dimension(n_orders) :: coupling_order
    integer :: color_id
    integer :: colorfac_id
    ! Propagator extension (only supported for massive vectors)
    ! 0: Usual propagator
    ! 1: Rxi-propagator part with pole @ M_V^2
    ! 2: Rxi-propagator part with pole @ \xi*M_V^2
    integer :: pext
  end type

  type Vertex
    type(Branch), allocatable :: branch(:)
    type(Branch), allocatable :: branchCT(:)
    type(Branch), allocatable :: branchR2(:)
  end type

  logical :: vertices_init = .false.
  logical :: vertices_filled = .false.
    interface createCurrent
      module procedure createCurrent2, createCurrent3, createCurrent4,         &
      createCurrent5, createCurrent6, createCurrent7, createCurrent8
    end interface 
  integer, parameter :: n_vertices = 669
  type(Vertex) :: vertices(0:n_vertices-1)

  ! Defines the maximum allowed number of legs participating at a vertex
  integer, parameter :: max_leg_multiplicity = 7

  integer, parameter :: FFaS0 = 1
  integer, parameter :: VSFSFa0 = 2
  integer, parameter :: VVSSVVS0 = 3
  integer, parameter :: VVSSVVS1 = 4
  integer, parameter :: VVSSVVS2 = 5
  integer, parameter :: SVVVS0 = 6
  integer, parameter :: SVVVS1 = 7
  integer, parameter :: FFaV0 = 8
  integer, parameter :: SVVVV0 = 9
  integer, parameter :: SVVVV1 = 10
  integer, parameter :: SVVVV2 = 11
  integer, parameter :: SVVVV3 = 12
  integer, parameter :: FaSFSV0 = 13
  integer, parameter :: SVSVSVV0 = 14
  integer, parameter :: SVSVSVV1 = 15
  integer, parameter :: SVSVSVV2 = 16
  integer, parameter :: VVSVSS0 = 17
  integer, parameter :: FFaVS0 = 18
  integer, parameter :: SVVVSVS0 = 19
  integer, parameter :: SVVVSVS1 = 20
  integer, parameter :: SVVVSVS2 = 21
  integer, parameter :: SVVVVSS0 = 22
  integer, parameter :: SVVVVSS1 = 23
  integer, parameter :: SVVVVSS2 = 24
  integer, parameter :: FaSVF0 = 25
  integer, parameter :: SSVVS0 = 26
  integer, parameter :: SSVVS1 = 27
  integer, parameter :: SSVVS2 = 28
  integer, parameter :: SSVVS3 = 29
  integer, parameter :: VVSV0 = 30
  integer, parameter :: VVSV1 = 31
  integer, parameter :: SVVSVVS0 = 32
  integer, parameter :: SVVSVVS1 = 33
  integer, parameter :: SVVSVVS2 = 34
  integer, parameter :: VVV0 = 35
  integer, parameter :: SVVSSVV0 = 36
  integer, parameter :: SVVSSVV1 = 37
  integer, parameter :: SVVSSVV2 = 38
  integer, parameter :: VVS0 = 39
  integer, parameter :: VVS1 = 40
  integer, parameter :: VSVVSVS0 = 41
  integer, parameter :: VSVVSVS1 = 42
  integer, parameter :: VSVVSVS2 = 43
  integer, parameter :: VSFFa0 = 44
  integer, parameter :: FFa0 = 45
  integer, parameter :: FaFVSS0 = 46
  integer, parameter :: FaFSV0 = 47
  integer, parameter :: VFaSFS0 = 48
  integer, parameter :: FaFSS0 = 49
  integer, parameter :: VUaU0 = 50
  integer, parameter :: VFSFa0 = 51
  integer, parameter :: FSFaVS0 = 52
  integer, parameter :: SVSSVVV0 = 53
  integer, parameter :: SVSSVVV1 = 54
  integer, parameter :: SVSSVVV2 = 55
  integer, parameter :: SVSVVS0 = 56
  integer, parameter :: SSFaFV0 = 57
  integer, parameter :: FaFS0 = 58
  integer, parameter :: FSVFaS0 = 59
  integer, parameter :: VSVSS0 = 60
  integer, parameter :: VSVSS1 = 61
  integer, parameter :: VSVSS2 = 62
  integer, parameter :: VSVSS3 = 63
  integer, parameter :: VSVSVSV0 = 64
  integer, parameter :: VSVSVSV1 = 65
  integer, parameter :: VSVSVSV2 = 66
  integer, parameter :: FaFV0 = 67
  integer, parameter :: SSSS0 = 68
  integer, parameter :: SFSVFa0 = 69
  integer, parameter :: FaSSVF0 = 70
  integer, parameter :: SFFa0 = 71
  integer, parameter :: SFaF0 = 72
  integer, parameter :: SVFSFa0 = 73
  integer, parameter :: SVV0 = 74
  integer, parameter :: SVV1 = 75
  integer, parameter :: FSFaV0 = 76
  integer, parameter :: FSFaS0 = 77
  integer, parameter :: SVSVVSV0 = 78
  integer, parameter :: SVSVVSV1 = 79
  integer, parameter :: SVSVVSV2 = 80
  integer, parameter :: VSSSVVV0 = 81
  integer, parameter :: VSSSVVV1 = 82
  integer, parameter :: VSSSVVV2 = 83
  integer, parameter :: VSFaFS0 = 84
  integer, parameter :: FSSVFa0 = 85
  integer, parameter :: VVSVSVS0 = 86
  integer, parameter :: VVSVSVS1 = 87
  integer, parameter :: VVSVSVS2 = 88
  integer, parameter :: FFaVSS0 = 89
  integer, parameter :: SVFFa0 = 90
  integer, parameter :: VVVSSS0 = 91
  integer, parameter :: VVVSSV0 = 92
  integer, parameter :: VSSFFa0 = 93
  integer, parameter :: FaSFV0 = 94
  integer, parameter :: SVFFaS0 = 95
  integer, parameter :: VSSVVSV0 = 96
  integer, parameter :: VSSVVSV1 = 97
  integer, parameter :: VSSVVSV2 = 98
  integer, parameter :: VSSSVV0 = 99
  integer, parameter :: FaVSFS0 = 100
  integer, parameter :: SVSFaF0 = 101
  integer, parameter :: FSFa0 = 102
  integer, parameter :: VSVSVVS0 = 103
  integer, parameter :: VSVSVVS1 = 104
  integer, parameter :: VSVSVVS2 = 105
  integer, parameter :: FVSFaS0 = 106
  integer, parameter :: VVSVVSS0 = 107
  integer, parameter :: VVSVVSS1 = 108
  integer, parameter :: VVSVVSS2 = 109
  integer, parameter :: VSSVVVS0 = 110
  integer, parameter :: VSSVVVS1 = 111
  integer, parameter :: VSSVVVS2 = 112
  integer, parameter :: VVSSSV0 = 113
  integer, parameter :: FSVFa0 = 114
  integer, parameter :: SSFVFa0 = 115
  integer, parameter :: SVSVVVS0 = 116
  integer, parameter :: SVSVVVS1 = 117
  integer, parameter :: SVSVVVS2 = 118
  integer, parameter :: VFaFS0 = 119
  integer, parameter :: FSSFaV0 = 120
  integer, parameter :: SVVVVS0 = 121
  integer, parameter :: VVSS0 = 122
  integer, parameter :: VVSS1 = 123
  integer, parameter :: FaSVSF0 = 124
  integer, parameter :: SFFaV0 = 125
  integer, parameter :: SFaVFS0 = 126
  integer, parameter :: VFaSF0 = 127
  integer, parameter :: VVVS0 = 128
  integer, parameter :: VVVS1 = 129
  integer, parameter :: VSFFaS0 = 130
  integer, parameter :: VVVV0 = 131
  integer, parameter :: SVSV0 = 132
  integer, parameter :: SVSV1 = 133
  integer, parameter :: VVSSSVV0 = 134
  integer, parameter :: VVSSSVV1 = 135
  integer, parameter :: VVSSSVV2 = 136
  integer, parameter :: VSVS0 = 137
  integer, parameter :: SFFaS0 = 138
  integer, parameter :: FaFSSV0 = 139
  integer, parameter :: SFSFa0 = 140
  integer, parameter :: SFaVSF0 = 141
  integer, parameter :: SVFaFS0 = 142
  integer, parameter :: SVSSV0 = 143
  integer, parameter :: SVSSV1 = 144
  integer, parameter :: SVSSV2 = 145
  integer, parameter :: SVSSV3 = 146
  integer, parameter :: VSSVVS0 = 147
  integer, parameter :: VSSSV0 = 148
  integer, parameter :: VSSSV1 = 149
  integer, parameter :: VSSSV2 = 150
  integer, parameter :: VSSSV3 = 151
  integer, parameter :: SFVSFa0 = 152
  integer, parameter :: SFaFVS0 = 153
  integer, parameter :: SSFFaV0 = 154
  integer, parameter :: SFaSF0 = 155
  integer, parameter :: SSVVSV0 = 156
  integer, parameter :: FaSF0 = 157
  integer, parameter :: FaSFS0 = 158
  integer, parameter :: SFaFV0 = 159
  integer, parameter :: SVFaSF0 = 160
  integer, parameter :: FFaSV0 = 161
  integer, parameter :: FaVSSF0 = 162
  integer, parameter :: SSVV0 = 163
  integer, parameter :: VFFaS0 = 164
  integer, parameter :: SSVVVVS0 = 165
  integer, parameter :: SSVVVVS1 = 166
  integer, parameter :: SSVVVVS2 = 167
  integer, parameter :: SSS0 = 168
  integer, parameter :: SSVSV0 = 169
  integer, parameter :: SSVSV1 = 170
  integer, parameter :: SSVSV2 = 171
  integer, parameter :: SFVFa0 = 172
  integer, parameter :: VSVSVS0 = 173
  integer, parameter :: FVSFa0 = 174
  integer, parameter :: SFFaSV0 = 175
  integer, parameter :: VFFa0 = 176
  integer, parameter :: VSVVSS0 = 177
  integer, parameter :: UaUV0 = 178
  integer, parameter :: SSVVVS0 = 179
  integer, parameter :: SVFaF0 = 180
  integer, parameter :: SFaVF0 = 181
  integer, parameter :: VSSFaF0 = 182
  integer, parameter :: VVVSS0 = 183
  integer, parameter :: VVVSS1 = 184
  integer, parameter :: VSSVSV0 = 185
  integer, parameter :: VVVSV0 = 186
  integer, parameter :: VVVSV1 = 187
  integer, parameter :: VVVSV2 = 188
  integer, parameter :: VVVSV3 = 189
  integer, parameter :: SSVSVV0 = 190
  integer, parameter :: FSSFa0 = 191
  integer, parameter :: VSVVV0 = 192
  integer, parameter :: VSVVV1 = 193
  integer, parameter :: VSVVV2 = 194
  integer, parameter :: VFFaSS0 = 195
  integer, parameter :: SSVVVSV0 = 196
  integer, parameter :: SSVVVSV1 = 197
  integer, parameter :: SSVVVSV2 = 198
  integer, parameter :: FSFaSV0 = 199
  integer, parameter :: SFaFS0 = 200
  integer, parameter :: FVSSFa0 = 201
  integer, parameter :: SVVS0 = 202
  integer, parameter :: SVVS1 = 203
  integer, parameter :: SFFaVS0 = 204
  integer, parameter :: FVFaS0 = 205
  integer, parameter :: VVSVSSV0 = 206
  integer, parameter :: VVSVSSV1 = 207
  integer, parameter :: VVSVSSV2 = 208
  integer, parameter :: SVSVS0 = 209
  integer, parameter :: SVSVS1 = 210
  integer, parameter :: SVSVS2 = 211
  integer, parameter :: SVSVS3 = 212
  integer, parameter :: VVVSVSS0 = 213
  integer, parameter :: VVVSVSS1 = 214
  integer, parameter :: VVVSVSS2 = 215
  integer, parameter :: SVSVSV0 = 216
  integer, parameter :: FFaSVS0 = 217
  integer, parameter :: SFVFaS0 = 218
  integer, parameter :: VSSVS0 = 219
  integer, parameter :: VSSVS1 = 220
  integer, parameter :: VSSVS2 = 221
  integer, parameter :: VSSVS3 = 222
  integer, parameter :: FVFa0 = 223
  integer, parameter :: FaF0 = 224
  integer, parameter :: SSSVVV0 = 225
  integer, parameter :: VVSVV0 = 226
  integer, parameter :: VVSVV1 = 227
  integer, parameter :: VVSVV2 = 228
  integer, parameter :: VFSFaS0 = 229
  integer, parameter :: SFaSVF0 = 230
  integer, parameter :: SVVSS0 = 231
  integer, parameter :: SVVSS1 = 232
  integer, parameter :: SVVSS2 = 233
  integer, parameter :: SSVFaF0 = 234
  integer, parameter :: SVVSV0 = 235
  integer, parameter :: SVVSV1 = 236
  integer, parameter :: FFaSS0 = 237
  integer, parameter :: FaSVFS0 = 238
  integer, parameter :: VVVSSSV0 = 239
  integer, parameter :: VVVSSSV1 = 240
  integer, parameter :: VVVSSSV2 = 241
  integer, parameter :: FVFaSS0 = 242
  integer, parameter :: SSSVV0 = 243
  integer, parameter :: SSSVV1 = 244
  integer, parameter :: SSSVV2 = 245
  integer, parameter :: SSSVV3 = 246
  integer, parameter :: VVVVSSS0 = 247
  integer, parameter :: VVVVSSS1 = 248
  integer, parameter :: VVVVSSS2 = 249
  integer, parameter :: FaVFSS0 = 250
  integer, parameter :: SFaSFV0 = 251
  integer, parameter :: SVVSSV0 = 252
  integer, parameter :: VSFaSF0 = 253
  integer, parameter :: SVVV0 = 254
  integer, parameter :: SVVV1 = 255
  integer, parameter :: VFaF0 = 256
  integer, parameter :: SSVVSVV0 = 257
  integer, parameter :: SSVVSVV1 = 258
  integer, parameter :: SSVVSVV2 = 259
  integer, parameter :: VVSSV0 = 260
  integer, parameter :: VVSSV1 = 261
  integer, parameter :: VVSSS0 = 262
  integer, parameter :: VVSSS1 = 263
  integer, parameter :: VVSSS2 = 264
  integer, parameter :: VVSSS3 = 265
  integer, parameter :: FFaSSV0 = 266
  integer, parameter :: UUaV0 = 267
  integer, parameter :: SVSFFa0 = 268
  integer, parameter :: VVVVSS0 = 269
  integer, parameter :: FaVFS0 = 270
  integer, parameter :: SSFaF0 = 271
  integer, parameter :: VSSVSVV0 = 272
  integer, parameter :: VSSVSVV1 = 273
  integer, parameter :: VSSVSVV2 = 274
  integer, parameter :: FSVSFa0 = 275
  integer, parameter :: VSVSSV0 = 276
  integer, parameter :: SFSFaV0 = 277
  integer, parameter :: SSFaVF0 = 278
  integer, parameter :: SSVSVVV0 = 279
  integer, parameter :: SSVSVVV1 = 280
  integer, parameter :: SSVSVVV2 = 281
  integer, parameter :: VVVVS0 = 282
  integer, parameter :: VVVVS1 = 283
  integer, parameter :: VVVVS2 = 284
  integer, parameter :: VVVVS3 = 285
  integer, parameter :: FaVF0 = 286
  integer, parameter :: FaSSFV0 = 287
  integer, parameter :: UaVU0 = 288
  integer, parameter :: VSVVVSS0 = 289
  integer, parameter :: VSVVVSS1 = 290
  integer, parameter :: VSVVVSS2 = 291
  integer, parameter :: VSFaF0 = 292
  integer, parameter :: SSSVVVV0 = 293
  integer, parameter :: SSSVVVV1 = 294
  integer, parameter :: SSSVVVV2 = 295
  integer, parameter :: SVVVSV0 = 296
  integer, parameter :: SVVVSS0 = 297
  integer, parameter :: FaFVS0 = 298
  integer, parameter :: VFaSSF0 = 299
  integer, parameter :: VSVSSVV0 = 300
  integer, parameter :: VSVSSVV1 = 301
  integer, parameter :: VSVSSVV2 = 302
  integer, parameter :: SVVVSSV0 = 303
  integer, parameter :: SVVVSSV1 = 304
  integer, parameter :: SVVVSSV2 = 305
  integer, parameter :: SSVFFa0 = 306
  integer, parameter :: VV0 = 307
  integer, parameter :: VV1 = 308
  integer, parameter :: FaVSF0 = 309
  integer, parameter :: FaSFVS0 = 310
  integer, parameter :: SVVSVS0 = 311
  integer, parameter :: VSVVSSV0 = 312
  integer, parameter :: VSVVSSV1 = 313
  integer, parameter :: VSVVSSV2 = 314
  integer, parameter :: FaFSVS0 = 315
  integer, parameter :: VFSSFa0 = 316
  integer, parameter :: FaSSF0 = 317
  integer, parameter :: VFaFSS0 = 318
  integer, parameter :: VSV0 = 319
  integer, parameter :: VSV1 = 320
  integer, parameter :: VVVSSVS0 = 321
  integer, parameter :: VVVSSVS1 = 322
  integer, parameter :: VVVSSVS2 = 323
  integer, parameter :: SSFFa0 = 324
  integer, parameter :: UVUa0 = 325
  integer, parameter :: VVSSVS0 = 326
  integer, parameter :: VSSV0 = 327
  integer, parameter :: VSSV1 = 328
  integer, parameter :: VVSSVSV0 = 329
  integer, parameter :: VVSSVSV1 = 330
  integer, parameter :: VVSSVSV2 = 331
  integer, parameter :: SVSSVV0 = 332
  integer, parameter :: SVVSVSV0 = 333
  integer, parameter :: SVVSVSV1 = 334
  integer, parameter :: SVVSVSV2 = 335
  integer, parameter :: SFaFSV0 = 336
  integer, parameter :: VUUa0 = 337
  complex(dp), parameter :: c1 = cmplx(-1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c1f = "-1/2"
  complex(dp), parameter :: c2 = cmplx(1d0/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c2f = "1/2"
  complex(dp), parameter :: c3 = cmplx(sq2/2d0, 0d0, kind=dp)
  character(len=20), parameter :: c3f = "sq2/2"
  complex(dp), parameter :: c4 = cmplx(-sq2/(2d0*Nc), 0d0, kind=dp)
  character(len=20), parameter :: c4f = "-sq2/(2*Nc)"
  complex(dp), parameter :: c5 = cmplx(0d0, sq2/2d0, kind=dp)
  character(len=20), parameter :: c5f = "i_*sq2/2"
  complex(dp), parameter :: c6 = cmplx(0d0, -sq2/2d0, kind=dp)
  character(len=20), parameter :: c6f = "-i_*sq2/2"
  complex(dp), parameter :: c7 = cmplx(1d0, 0d0, kind=dp)
  character(len=20), parameter :: c7f = "1"
  complex(dp), parameter :: c8 = cmplx(-1d0/Nc, 0d0, kind=dp)
  character(len=20), parameter :: c8f = "-1/Nc"
  complex(dp), dimension(1:8) :: colorcoeffs = (/c1, c2, c3, c4, c5, c6, c7, c8&
                                               /)
  character(len=20), dimension(1:8) :: colorcoeffs_form = (/c1f, c2f, c3f, c4f &
                 , c5f, c6f, c7f, c8f/)
  integer, dimension(1:337) :: lorentz_rank = (/-1, -1, 0, 0, 0, 0, -1, 0, 0, -&
                                              1, -1, -1, -1, 0, 0, 0, 1, -1, 0 &
                                              , 0, 0, 0, 0, 0, -1, 0, -1, -1, -&
                                              1, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1 &
                                              , -1, 0, 0, 0, -1, -1, -1, -1, -1&
                                              , -1, 1, -1, -1, 0, 0, 0, 0, -1  &
                                              , -1, -1, 1, -1, -1, -1, 0, 0, 0 &
                                              , 0, 0, -1, -1, -1, -1, -1, 1, -1&
                                              , -1, -1, 0, 0, 0, 0, 0, 0, -1, -&
                                              1, 0, 0, 0, -1, -1, 1, 0, -1, -1 &
                                              , -1, 0, 0, 0, 1, -1, -1, -1, 0, &
                                              0, 0, -1, 0, 0, 0, 0, 0, 0, 1, -1&
                                              , -1, 0, 0, 0, -1, -1, 0, 1, -1  &
                                              , -1, -1, -1, -1, 1, -1, -1, 0, 1&
                                              , -1, 0, 0, 0, -1, -1, -1, -1, -1&
                                              , -1, 1, -1, -1, -1, 1, 2, -1, -1&
                                              , -1, -1, -1, -1, -1, 1, -1, -1  &
                                              , -1, -1, -1, -1, -1, -1, 0, 0, 0&
                                              , 0, 1, -1, -1, -1, 1, -1, -1, 1 &
                                              , 1, 1, 0, -1, -1, -1, 1, -1, 1, &
                                              0, -1, -1, -1, 1, -1, -1, -1, -1 &
                                              , -1, 0, 0, 0, -1, -1, -1, 0, -1 &
                                              , -1, -1, 0, 0, 0, 0, -1, -1, -1 &
                                              , 0, 0, 0, 1, -1, -1, 1, -1, -1  &
                                              , -1, 1, -1, 1, -1, -1, -1, -1, -&
                                              1, 0, -1, -1, -1, 1, -1, -1, -1, &
                                              0, 0, 0, -1, 1, -1, -1, -1, 0, 0 &
                                              , 0, -1, -1, 1, -1, 1, -1, 1, 0, &
                                              0, 0, 1, -1, 1, -1, -1, -1, -1, 1&
                                              , -1, 0, -1, -1, 0, 0, 0, -1, 1  &
                                              , -1, -1, 0, 0, 0, 0, -1, -1, -1 &
                                              , 1, -1, 1, 0, 0, 0, -1, 0, 0, 0 &
                                              , 0, 0, -1, -1, 0, 0, 0, 0, 0, 0 &
                                              , -1, -1, -1, -1, -1, 0, 0, 0, 0 &
                                              , -1, -1, -1, -1, 2, -1, 0, 0, 0 &
                                              , -1, 1, 1, 2, -1, 0, 0, 0, 1, 0 &
                                              , 0, 0, -1, 1/)
  integer, parameter :: VVVS_heft = 140
  integer, parameter :: SVSV_heft = 104
  integer, parameter :: SVVVS_heft = 156
  integer, parameter :: SVVVV_heft = 114
  integer, parameter :: VVVVSS_heft = 120
  integer, parameter :: VVSSV_heft = 153
  integer, parameter :: VVVVS_heft = 110
  integer, parameter :: VVSV_heft = 141
  integer, parameter :: VVS_heft = 97
  integer, parameter :: VVVSS_heft = 150
  integer, parameter :: VVVSV_heft = 111
  integer, parameter :: SVVS_heft = 103
  integer, parameter :: VSV_heft = 98
  integer, parameter :: VSSV_heft = 102
  integer, parameter :: SVV_heft = 99
  integer, parameter :: VVVSSV_heft = 122
  integer, parameter :: SVVSV_heft = 157
  integer, parameter :: SVVVSV_heft = 131
  integer, parameter :: SVVVVS_heft = 130
  integer, parameter :: VVSS_heft = 100
  integer, parameter :: SVVV_heft = 143
  integer, dimension(1:337) :: recola_base = (/0, 0, 0, 0, 0, SVVVS_heft, 0, 15&
                                             , SVVVV_heft, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, VVSV_heft, 0, 0, 0, 0, 12 &
                                             , 0, 0, 0, VVS_heft, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 162, 0, 0, 0, 0, &
                                             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                             , 16, -1, 0, 0, 0, 0, 0, SVV_heft &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, VVVSSV_heft, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, SVVVVS_heft,     &
                                             VVSS_heft, 0, 0, 0, 0, 0,         &
                                             VVVS_heft, 0, 0, -11, SVSV_heft, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 1, 0, 0, 0, 0, 0, 0, 0, 33, 0&
                                             , 164, 0, 0, 0, 0, VVVSS_heft, 0, &
                                             0, VVVSV_heft, 0, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0,         &
                                             SVVS_heft, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 31, 0, 0, 0, 0, 0, 0, 0, 0, 0&
                                             , 0, 0, SVVSV_heft, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, SVVV_heft, 0, 53, 0, 0, 0 &
                                             , VVSSV_heft, 0, 0, 0, 0, 0, 0,   &
                                             161, 0, VVVVSS_heft, 0, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, VVVVS_heft &
                                             , 0, 0, 0, 51, 0, 166, 0, 0, 0, 0 &
                                             , 0, 0, 0, SVVVSV_heft, 0, 0, 0, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 &
                                             , 0, 0, 0, 0, 0, 0, 0, VSV_heft, 0&
                                             , 0, 0, 0, 0, 165, 0, VSSV_heft, 0&
                                             , 0, 0, 0, 0, 0, 0, 0, 0, 163/)
  integer, dimension(1:337) :: propagator_extension = (/0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0, 0, 0, 0, 0, 0, 0, 0 &
                                                      , 0/)
  character(len=*), parameter :: model_timestamp = "Thu Mar 21 23:28:27 2019"

  contains
    function get_timestamp_mdl() result(ret_timestamp)
      character(len=len(model_timestamp)) :: ret_timestamp
      ret_timestamp = model_timestamp
    end function get_timestamp_mdl

    function get_lorentz_rank_mdl(l_id) result(ret_lorentz_rank)
      integer, intent(in) :: l_id
      integer :: ret_lorentz_rank
      ret_lorentz_rank = lorentz_rank(l_id)
    end function get_lorentz_rank_mdl

    function get_propagator_extension_mdl(l_id) result(ret_pext)
      integer, intent(in) :: l_id
      integer :: ret_pext
      ret_pext = propagator_extension(l_id)
    end function get_propagator_extension_mdl

    function get_max_leg_multiplicity_mdl() result(ret_max_leg_multiplicity)
      integer :: ret_max_leg_multiplicity
      ret_max_leg_multiplicity = max_leg_multiplicity
    end function get_max_leg_multiplicity_mdl

    function is_model_init_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_init
    end function is_model_init_mdl

    function is_model_filled_mdl() result(ret_init)
      logical :: ret_init
      ret_init = vertices_filled
    end function is_model_filled_mdl

    subroutine get_vertex3_mdl(p1, p2, p3, vertex_ret)
      integer, intent(out) :: vertex_ret
      integer, intent(in)  :: p1, p2, p3
      integer(kind=8)      :: config
      integer              :: address
      logical              :: found_vertex

      call createParticleConfig(p1, p2, p3, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if

    end subroutine get_vertex3_mdl

    subroutine get_vertex4_mdl(p1, p2, p3, p4, vertex_ret)
      integer, intent(out) :: vertex_ret
      integer, intent(in)  :: p1, p2, p3, p4
      integer(kind=8)      :: config
      integer              :: address
      logical              :: found_vertex

      call createParticleConfig(p1, p2, p3, p4, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if
    end subroutine get_vertex4_mdl

    subroutine get_vertexN_mdl(particles, particleOut, vertex_ret)
      integer, intent(out)              :: vertex_ret
      integer, dimension(:), intent(in) :: particles
      integer, intent(in)               :: particleOut
      integer(kind=8)                   :: config
      integer                           :: address
      logical                           :: found_vertex

      call createParticleConfig(particles, particleOut, config)
      call dict(config, 1, address, found_vertex, 0)
      if(found_vertex) then
        vertex_ret = dict_get_value(address)
      else
        vertex_ret = -1
      end if

    end subroutine get_vertexN_mdl

    subroutine av(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branch(bid)%color_id = cid
      vertices(vid)%branch(bid)%colorfac_id = cfid
      vertices(vid)%branch(bid)%pext = pext
      vertices(vid)%branch(bid)%lorentz_id = lid
      vertices(vid)%branch(bid)%perm = perm
      vertices(vid)%branch(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branch(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branch(bid)%couplings(i) = cop(i)
      end do
    end subroutine av

    subroutine avCT(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchCT(bid)%color_id = cid
      vertices(vid)%branchCT(bid)%colorfac_id = cfid
      vertices(vid)%branchCT(bid)%pext = pext
      vertices(vid)%branchCT(bid)%lorentz_id = lid
      vertices(vid)%branchCT(bid)%perm = perm
      vertices(vid)%branchCT(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchCT(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchCT(bid)%couplings(i) = cop(i)
      end do
    end subroutine avCT

    subroutine avR2(vid,bid,cid,cfid,pext,lid,perm,co,cop)
      integer, intent(in) :: vid,bid,cid,cfid,pext,lid,perm
      integer, dimension(0:), intent(in) :: co, cop
      integer scop,i
      vertices(vid)%branchR2(bid)%color_id = cid
      vertices(vid)%branchR2(bid)%colorfac_id = cfid
      vertices(vid)%branchR2(bid)%pext = pext
      vertices(vid)%branchR2(bid)%lorentz_id = lid
      vertices(vid)%branchR2(bid)%perm = perm
      vertices(vid)%branchR2(bid)%coupling_order = co
      scop = size(cop)
      allocate(vertices(vid)%branchR2(bid)%couplings(0:scop-1))
      do i = 0, scop-1
        vertices(vid)%branchR2(bid)%couplings(i) = cop(i)
      end do
    end subroutine avR2

    function get_colourfac_mdl(cfid)
      integer, intent(in) :: cfid
      complex(dp) :: get_colourfac_mdl
      get_colourfac_mdl = colorcoeffs(cfid)
    end function get_colourfac_mdl

    function get_colourfac_form_mdl(cfid)
      integer, intent(in) :: cfid
      character(len=20) :: get_colourfac_form_mdl
      get_colourfac_form_mdl = colorcoeffs_form(cfid)
    end function get_colourfac_form_mdl

  function get_recola_base_mdl(l_id) result(ret_rb)
      integer, intent(in) :: l_id
      integer :: ret_rb
      ret_rb = recola_base(l_id)
    end function get_recola_base_mdl

  subroutine get_vertex_branch_status_mdl(vx,xlp,branch_exists)
    integer, intent(in)  :: vx,xlp
    logical, intent(out) ::branch_exists
    select case (xlp)
    case(TreeBranch:LoopBranch)
      branch_exists = allocated(vertices(vx)%branch)
    case(CTBranch)
      branch_exists = allocated(vertices(vx)%branchCT)
    case(R2Branch)
      branch_exists = allocated(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_branch_status_mdl

  subroutine get_vertex_colour_branches_mdl(vx,xlp,cfids,nbranches)
    integer, allocatable, intent(out) :: cfids(:)
    integer, intent(out) :: nbranches
    integer, intent(in)  :: vx,xlp
    select case (xlp)
    case (TreeBranch:LoopBranch)
      allocate(cfids(size(vertices(vx)%branch)))
      cfids(:) = vertices(vx)%branch(0:)%color_id
      nbranches = size(vertices(vx)%branch)
    case(CTBranch)
      allocate(cfids(size(vertices(vx)%branchCT)))
      cfids(:) = vertices(vx)%branchCT(0:)%color_id
      nbranches = size(vertices(vx)%branchCT)
    case(R2Branch)
      allocate(cfids(size(vertices(vx)%branchR2)))
      cfids(:) = vertices(vx)%branchR2(0:)%color_id
      nbranches = size(vertices(vx)%branchR2)
    end select
  end subroutine get_vertex_colour_branches_mdl

  subroutine get_vertex_pext_mdl(vx,xlp,br,pext)
    integer, intent(in) :: vx,xlp,br
    integer, intent(out) :: pext
    select case (xlp)
    case (TreeBranch:LoopBranch)
      pext = vertices(vx)%branch(br)%pext
    case(CTBranch)
      pext = vertices(vx)%branchCT(br)%pext
    case(R2Branch)
      pext = vertices(vx)%branchR2(br)%pext
    end select
  end subroutine get_vertex_pext_mdl

  subroutine get_vertex_order_increase_mdl(vx,xlp,br,orderInc)
    integer, intent(in) :: vx,xlp,br
    integer, dimension(:), intent(out) :: orderInc
    select case (xlp)
    case (TreeBranch:LoopBranch)
      orderInc = vertices(vx)%branch(br)%coupling_order
    case(CTBranch)
      orderInc = vertices(vx)%branchCT(br)%coupling_order
    case(R2Branch)
      orderInc = vertices(vx)%branchR2(br)%coupling_order
    end select
  end subroutine get_vertex_order_increase_mdl

  subroutine get_vertex_lorentz_type_mdl(vx,xlp,br,lid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: lid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      lid = vertices(vx)%branch(br)%lorentz_id
    case(CTBranch)
      lid = vertices(vx)%branchCT(br)%lorentz_id
    case(R2Branch)
      lid = vertices(vx)%branchR2(br)%lorentz_id
    end select
  end subroutine get_vertex_lorentz_type_mdl

  subroutine get_vertex_perm_mdl(vx,xlp,br,perm)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: perm
    select case (xlp)
    case (TreeBranch:LoopBranch)
      perm = vertices(vx)%branch(br)%perm
    case(CTBranch)
      perm = vertices(vx)%branchCT(br)%perm
    case(R2Branch)
      perm = vertices(vx)%branchR2(br)%perm
    end select
  end subroutine get_vertex_perm_mdl

  subroutine get_vertex_colour_id_mdl(vx,xlp,br,cid)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cid
    select case (xlp)
    case (TreeBranch:LoopBranch)
      cid = vertices(vx)%branch(br)%colorfac_id
    case(CTBranch)
      cid = vertices(vx)%branchCT(br)%colorfac_id
    case(R2Branch)
      cid = vertices(vx)%branchR2(br)%colorfac_id
    end select
  end subroutine get_vertex_colour_id_mdl

  function is_zero_coupling_vertex_mdl(vx,xlp,br) result(zero)
    use class_couplings, only: get_coupling_value_mdl
    use input_mdl, only: zerocut
    integer, intent(in)  :: vx,xlp,br
    logical              :: zero
    real(dp)             :: maxvalue
    integer              :: i

    zero = .false.
    select case (xlp)
    case (TreeBranch:LoopBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branch(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branch(br)%couplings(i-&
                1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(CTBranch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchCT(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchCT(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    case(R2Branch)
      maxvalue = 0d0
      do i = 1,size(vertices(vx)%branchR2(br)%couplings)
        maxvalue = maxvalue + &
                abs(get_coupling_value_mdl(vertices(vx)%branchR2(br)%couplings(&
                i-1)))
      end do
      if (maxvalue/zerocut .lt. 100d0) then
        zero = .true.
      end if
    end select
  end function is_zero_coupling_vertex_mdl

  subroutine get_vertex_ncouplings_mdl(vx,xlp,br,ncouplings)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: ncouplings
    select case(xlp)
    case(TreeBranch, LoopBranch)
      ncouplings = size(vertices(vx)%branch(br)%couplings)
    case (CTBranch)
      ncouplings = size(vertices(vx)%branchCT(br)%couplings)
    case (R2Branch)
      ncouplings = size(vertices(vx)%branchR2(br)%couplings)
    end select
  end subroutine get_vertex_ncouplings_mdl

  subroutine get_vertex_coupling_ids_mdl(vx,xlp,br,cids)
    integer, intent(in)  :: vx,xlp,br
    integer, intent(out) :: cids(:)
    select case(xlp)
    case(TreeBranch, LoopBranch)
      cids(:) = vertices(vx)%branch(br)%couplings(:)
    case (CTBranch)
      cids(:) = vertices(vx)%branchCT(br)%couplings(:)
    case (R2Branch)
      cids(:) = vertices(vx)%branchR2(br)%couplings(:)
    end select
  end subroutine get_vertex_coupling_ids_mdl

    subroutine createCurrent2(P1, P2, id, check)
      integer, intent(in) :: P1, P2, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent2

    subroutine createCurrent3(P1, P2, P3, id, check)
      integer, intent(in) :: P1, P2, P3, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent3

    subroutine createCurrent4(P1, P2, P3, P4, id, check)
      integer, intent(in) :: P1, P2, P3, P4, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent4

    subroutine createCurrent5(P1, P2, P3, P4, P5, id, check)
      integer, intent(in) :: P1, P2, P3, P4, P5, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent5

    subroutine createCurrent6(P1, P2, P3, P4, P5, P6, id, check)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent6

    subroutine createCurrent7(P1, P2, P3, P4, P5, P6, P7, id, check)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, P7, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, P7, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent7

    subroutine createCurrent8(P1, P2, P3, P4, P5, P6, P7, P8, id, check)
      integer, intent(in) :: P1, P2, P3, P4, P5, P6, P7, P8, id
      logical, intent(in) :: check
      integer(kind=8) :: key
      integer :: res,address
      logical :: found
      call createParticleConfig(P1, P2, P3, P4, P5, P6, P7, P8, key)
      if (.not. check) then
        call dict(key, 2, address, found, id)
      else
        call dict(key, 1, address, found, 0)
        res = dict_get_value(address)
        write (*,*) "out=", res, found, address, "shouldbe = ", id
        if (res /= id) then
          stop 9
        end if 
      end if 
    end subroutine createCurrent8

    subroutine init_vertices_mdl()
      use constants_mdl
      use input_mdl, only: nx, error_mdl

      ! check if modelfile is compatible to drivers
      if (model_timestamp .ne. driver_timestamp) then
        call error_mdl("Model corrupted. Driver timestamp mismatch.", &
                      where='init_vertices')
      end if

      vertices_init = .true.
      call fill_couplings_mdl
      call init_CTparameters_mdl(.true.)
      call fill_CTcouplings_mdl
      call dict_init(1327)
      call createCurrents(.false.)

    end subroutine init_vertices_mdl

    subroutine createCurrents(check)
      logical, intent(in) :: check
      call createCurrent(P_g, P_dAnti, P_H, P_d, P_H, 639, check)
      call createCurrent(P_dAnti, P_d, P_g, P_H, P_H, 232, check)
      call createCurrent(P_H, P_H, P_d, P_g, P_dAnti, 575, check)
      call createCurrent(P_H, P_g, P_H, P_H, P_g, 51, check)
      call createCurrent(P_H, P_dAnti, P_H, P_g, P_d, 612, check)
      call createCurrent(P_b, P_g, P_H, P_H, P_bAnti, 260, check)
      call createCurrent(P_H, P_g, P_H, P_u, P_uAnti, 481, check)
      call createCurrent(P_bAnti, P_H, P_g, P_H, P_b, 627, check)
      call createCurrent(P_d, P_H, P_g, P_dAnti, P_H, 662, check)
      call createCurrent(P_c, P_H, P_H, P_cAnti, P_g, 314, check)
      call createCurrent(P_d, P_dAnti, 153, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_g, P_H, P_H, 50, check)
      call createCurrent(P_uAnti, P_H, P_u, P_g, P_H, 508, check)
      call createCurrent(P_H, P_s, P_sAnti, P_g, P_H, 410, check)
      call createCurrent(P_d, P_dAnti, P_H, P_g, 258, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_g, P_H, 35, check)
      call createCurrent(P_g, P_H, P_uAnti, P_H, P_u, 178, check)
      call createCurrent(P_cAnti, P_g, P_c, P_H, P_H, 226, check)
      call createCurrent(P_u, P_g, P_H, P_uAnti, P_H, 568, check)
      call createCurrent(P_H, P_b, P_g, P_bAnti, 429, check)
      call createCurrent(P_H, P_b, P_bAnti, P_H, 236, check)
      call createCurrent(P_uAnti, P_u, P_H, P_H, P_g, 347, check)
      call createCurrent(P_s, P_H, P_H, P_sAnti, P_g, 395, check)
      call createCurrent(P_cAnti, P_H, P_g, P_c, P_H, 454, check)
      call createCurrent(P_H, P_c, P_H, P_cAnti, 222, check)
      call createCurrent(P_s, P_sAnti, P_H, 309, check)
      call createCurrent(P_H, P_cAnti, P_H, P_c, 579, check)
      call createCurrent(P_H, P_g, P_c, P_cAnti, P_H, 430, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_H, P_g, P_g, 101, check)
      call createCurrent(P_g, P_cAnti, P_c, 117, check)
      call createCurrent(P_H, P_dAnti, P_g, P_H, P_d, 373, check)
      call createCurrent(P_H, P_s, P_g, P_H, P_sAnti, 462, check)
      call createCurrent(P_H, P_H, P_dAnti, P_d, 394, check)
      call createCurrent(P_H, P_c, P_H, P_cAnti, P_g, 275, check)
      call createCurrent(P_g, P_H, P_u, P_uAnti, P_H, 603, check)
      call createCurrent(P_H, P_g, P_b, P_H, P_bAnti, 583, check)
      call createCurrent(P_uAnti, P_u, 148, check)
      call createCurrent(P_sAnti, P_g, P_s, 146, check)
      call createCurrent(P_g, P_H, P_H, P_bAnti, P_b, 319, check)
      call createCurrent(P_H, P_g, P_dAnti, P_d, P_H, 434, check)
      call createCurrent(P_H, P_u, P_uAnti, P_g, 629, check)
      call createCurrent(P_c, P_g, P_cAnti, P_H, 652, check)
      call createCurrent(P_H, P_c, P_g, P_cAnti, 376, check)
      call createCurrent(P_s, P_g, P_sAnti, 69, check)
      call createCurrent(P_sAnti, P_H, P_H, P_s, P_g, 372, check)
      call createCurrent(P_d, P_g, P_H, P_H, P_dAnti, 535, check)
      call createCurrent(P_g, P_b, P_bAnti, P_H, 172, check)
      call createCurrent(P_d, P_H, P_H, P_dAnti, 227, check)
      call createCurrent(P_H, P_u, P_uAnti, P_g, P_H, 472, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_g, P_H, 132, check)
      call createCurrent(P_H, P_H, P_H, P_g, P_g, P_g, 77, check)
      call createCurrent(P_c, P_H, P_cAnti, P_g, P_H, 324, check)
      call createCurrent(P_H, P_g, P_sAnti, P_s, P_H, 493, check)
      call createCurrent(P_H, P_H, P_s, P_sAnti, P_g, 534, check)
      call createCurrent(P_H, P_g, P_dAnti, P_H, P_d, 619, check)
      call createCurrent(P_g, P_H, P_dAnti, P_d, 445, check)
      call createCurrent(P_s, P_H, P_g, P_sAnti, P_H, 515, check)
      call createCurrent(P_b, P_H, P_H, P_bAnti, P_g, 474, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_H, 125, check)
      call createCurrent(P_dAnti, P_d, 158, check)
      call createCurrent(P_uAnti, P_u, P_H, P_g, P_H, 289, check)
      call createCurrent(P_b, P_H, P_H, P_bAnti, 233, check)
      call createCurrent(P_g, P_H, P_g, 116, check)
      call createCurrent(P_uAnti, P_u, P_g, P_H, P_H, 262, check)
      call createCurrent(P_uAnti, P_H, P_u, P_g, 256, check)
      call createCurrent(P_H, P_H, P_g, P_cAnti, P_c, 163, check)
      call createCurrent(P_d, P_dAnti, P_H, P_H, P_g, 257, check)
      call createCurrent(P_H, P_g, P_s, P_sAnti, 174, check)
      call createCurrent(P_g, P_d, P_dAnti, 1, check)
      call createCurrent(P_b, P_bAnti, P_H, P_H, P_g, 197, check)
      call createCurrent(P_H, P_uAnti, P_H, P_g, P_u, 540, check)
      call createCurrent(P_cAnti, P_g, P_c, 17, check)
      call createCurrent(P_H, P_g, P_uAnti, P_u, P_H, 477, check)
      call createCurrent(P_c, P_g, P_cAnti, P_H, P_H, 498, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_H, P_g, 7, check)
      call createCurrent(P_b, P_bAnti, P_g, P_H, P_H, 458, check)
      call createCurrent(P_g, P_H, P_b, P_bAnti, P_H, 318, check)
      call createCurrent(P_g, P_sAnti, P_H, P_s, 370, check)
      call createCurrent(P_bAnti, P_b, P_g, 123, check)
      call createCurrent(P_g, P_d, P_dAnti, P_H, 422, check)
      call createCurrent(P_H, P_s, P_sAnti, P_H, 254, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_H, P_g, P_H, 41, check)
      call createCurrent(P_g, P_g, P_H, 47, check)
      call createCurrent(P_sAnti, P_H, P_g, P_H, P_s, 566, check)
      call createCurrent(P_H, P_H, P_sAnti, P_s, 590, check)
      call createCurrent(P_g, P_s, P_sAnti, P_H, P_H, 406, check)
      call createCurrent(P_g, P_g, P_g, P_g, P_H, 38, check)
      call createCurrent(P_u, P_uAnti, P_H, P_g, P_H, 529, check)
      call createCurrent(P_g, P_cAnti, P_c, P_H, 538, check)
      call createCurrent(P_H, P_H, P_uAnti, P_u, 349, check)
      call createCurrent(P_H, P_cAnti, P_c, P_g, P_H, 502, check)
      call createCurrent(P_H, P_sAnti, P_s, P_H, 668, check)
      call createCurrent(P_H, P_b, P_g, P_H, P_bAnti, 450, check)
      call createCurrent(P_sAnti, P_s, P_H, P_H, P_g, 217, check)
      call createCurrent(P_u, P_H, P_g, P_uAnti, 329, check)
      call createCurrent(P_g, P_H, P_u, P_H, P_uAnti, 277, check)
      call createCurrent(P_H, P_g, P_g, P_g, 122, check)
      call createCurrent(P_H, P_cAnti, P_c, 310, check)
      call createCurrent(P_c, P_cAnti, P_H, P_g, P_H, 547, check)
      call createCurrent(P_ghG, P_ghGAnti, P_g, 72, check)
      call createCurrent(P_H, P_H, P_uAnti, P_g, P_u, 377, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_H, P_g, 93, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_H, 49, check)
      call createCurrent(P_dAnti, P_H, P_d, P_g, P_H, 461, check)
      call createCurrent(P_g, P_H, P_d, P_dAnti, P_H, 469, check)
      call createCurrent(P_H, P_H, P_cAnti, P_c, 187, check)
      call createCurrent(P_cAnti, P_H, P_c, P_g, 221, check)
      call createCurrent(P_sAnti, P_s, 152, check)
      call createCurrent(P_s, P_sAnti, P_H, P_g, 444, check)
      call createCurrent(P_g, P_H, P_bAnti, P_H, P_b, 561, check)
      call createCurrent(P_H, P_g, P_uAnti, P_H, P_u, 557, check)
      call createCurrent(P_g, P_H, P_b, P_H, P_bAnti, 220, check)
      call createCurrent(P_H, P_dAnti, P_H, P_d, 512, check)
      call createCurrent(P_H, P_c, P_cAnti, P_g, P_H, 407, check)
      call createCurrent(P_H, P_g, P_cAnti, P_H, P_c, 415, check)
      call createCurrent(P_H, P_uAnti, P_u, 435, check)
      call createCurrent(P_s, P_H, P_sAnti, P_H, P_g, 632, check)
      call createCurrent(P_H, P_d, P_g, P_dAnti, P_H, 638, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_H, P_H, P_g, 36, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_g, 64, check)
      call createCurrent(P_H, P_dAnti, P_d, P_H, P_g, 601, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_H, P_g, P_g, 73, check)
      call createCurrent(P_dAnti, P_H, P_H, P_d, P_g, 555, check)
      call createCurrent(P_H, P_H, P_g, P_sAnti, P_s, 541, check)
      call createCurrent(P_H, P_g, P_b, P_bAnti, P_H, 413, check)
      call createCurrent(P_g, P_d, P_H, P_dAnti, 492, check)
      call createCurrent(P_H, P_s, P_sAnti, P_H, P_g, 183, check)
      call createCurrent(P_H, P_g, P_s, P_H, P_sAnti, 249, check)
      call createCurrent(P_g, P_H, P_H, P_g, 42, check)
      call createCurrent(P_H, P_d, P_H, P_dAnti, P_g, 513, check)
      call createCurrent(P_H, P_g, P_dAnti, P_d, 528, check)
      call createCurrent(P_H, P_sAnti, P_s, 334, check)
      call createCurrent(P_H, P_H, P_g, P_g, 11, check)
      call createCurrent(P_H, P_dAnti, P_d, P_g, P_H, 487, check)
      call createCurrent(P_sAnti, P_s, P_H, P_g, P_H, 185, check)
      call createCurrent(P_c, P_H, P_cAnti, P_g, 357, check)
      call createCurrent(P_u, P_H, P_uAnti, P_g, P_H, 320, check)
      call createCurrent(P_c, P_H, P_H, P_g, P_cAnti, 384, check)
      call createCurrent(P_sAnti, P_s, P_H, 294, check)
      call createCurrent(P_H, P_g, P_c, P_H, P_cAnti, 460, check)
      call createCurrent(P_g, P_u, P_uAnti, 52, check)
      call createCurrent(P_ghGAnti, P_g, P_ghG, 99, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_g, 95, check)
      call createCurrent(P_H, P_H, P_d, P_dAnti, 432, check)
      call createCurrent(P_H, P_d, P_H, P_dAnti, 405, check)
      call createCurrent(P_dAnti, P_H, P_g, P_H, P_d, 573, check)
      call createCurrent(P_H, P_g, P_H, P_sAnti, P_s, 641, check)
      call createCurrent(P_g, P_H, P_s, P_sAnti, P_H, 521, check)
      call createCurrent(P_H, P_u, P_g, P_H, P_uAnti, 580, check)
      call createCurrent(P_dAnti, P_g, P_d, 8, check)
      call createCurrent(P_d, P_H, P_dAnti, P_g, 327, check)
      call createCurrent(P_dAnti, P_g, P_H, P_H, P_d, 411, check)
      call createCurrent(P_g, P_H, P_H, P_H, P_g, P_g, 87, check)
      call createCurrent(P_g, P_bAnti, P_H, P_b, P_H, 179, check)
      call createCurrent(P_d, P_g, P_H, P_dAnti, 338, check)
      call createCurrent(P_bAnti, P_H, P_H, P_g, P_b, 388, check)
      call createCurrent(P_d, P_H, P_dAnti, P_g, P_H, 255, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_g, P_g, P_H, 59, check)
      call createCurrent(P_cAnti, P_H, P_H, P_c, P_g, 456, check)
      call createCurrent(P_uAnti, P_g, P_u, P_H, P_H, 507, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_H, 88, check)
      call createCurrent(P_uAnti, P_u, P_g, P_H, 615, check)
      call createCurrent(P_H, P_H, P_u, P_uAnti, P_g, 378, check)
      call createCurrent(P_H, P_c, P_cAnti, P_H, P_g, 527, check)
      call createCurrent(P_H, P_H, P_bAnti, P_b, P_g, 548, check)
      call createCurrent(P_H, P_c, P_H, P_g, P_cAnti, 582, check)
      call createCurrent(P_H, P_g, P_H, P_uAnti, P_u, 315, check)
      call createCurrent(P_g, P_u, P_H, P_uAnti, P_H, 539, check)
      call createCurrent(P_u, P_uAnti, P_H, 475, check)
      call createCurrent(P_H, P_cAnti, P_H, P_c, P_g, 463, check)
      call createCurrent(P_u, P_H, P_uAnti, P_H, P_g, 485, check)
      call createCurrent(P_H, P_H, P_g, P_uAnti, P_u, 459, check)
      call createCurrent(P_g, P_H, P_d, P_dAnti, 269, check)
      call createCurrent(P_g, P_H, P_dAnti, P_d, P_H, 203, check)
      call createCurrent(P_H, P_H, P_c, P_cAnti, P_g, 191, check)
      call createCurrent(P_bAnti, P_H, P_g, P_b, 571, check)
      call createCurrent(P_H, P_g, P_H, P_b, P_bAnti, 341, check)
      call createCurrent(P_c, P_cAnti, P_H, 374, check)
      call createCurrent(P_cAnti, P_c, P_g, 92, check)
      call createCurrent(P_H, P_H, P_H, 96, check)
      call createCurrent(P_b, P_bAnti, P_H, 298, check)
      call createCurrent(P_c, P_cAnti, P_H, P_H, P_g, 198, check)
      call createCurrent(P_d, P_H, P_dAnti, P_H, P_g, 570, check)
      call createCurrent(P_d, P_dAnti, P_g, 106, check)
      call createCurrent(P_g, P_H, P_H, P_c, P_cAnti, 495, check)
      call createCurrent(P_g, P_uAnti, P_H, P_u, P_H, 661, check)
      call createCurrent(P_b, P_g, P_H, P_bAnti, P_H, 397, check)
      call createCurrent(P_H, P_u, P_g, P_uAnti, 206, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_H, P_g, P_g, 140, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_H, P_g, 29, check)
      call createCurrent(P_g, P_d, P_dAnti, P_H, P_H, 599, check)
      call createCurrent(P_u, P_H, P_g, P_H, P_uAnti, 195, check)
      call createCurrent(P_H, P_d, P_dAnti, P_H, 533, check)
      call createCurrent(P_b, P_g, P_bAnti, P_H, P_H, 634, check)
      call createCurrent(P_H, P_sAnti, P_H, P_s, 385, check)
      call createCurrent(P_H, P_g, P_H, P_bAnti, P_b, 248, check)
      call createCurrent(P_s, P_g, P_H, P_H, P_sAnti, 186, check)
      call createCurrent(P_H, P_cAnti, P_g, P_c, 516, check)
      call createCurrent(P_dAnti, P_H, P_H, P_g, P_d, 328, check)
      call createCurrent(P_H, P_uAnti, P_H, P_u, 556, check)
      call createCurrent(P_dAnti, P_H, P_d, 170, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_g, P_H, 24, check)
      call createCurrent(P_H, P_d, P_g, P_H, P_dAnti, 390, check)
      call createCurrent(P_b, P_H, P_H, P_g, P_bAnti, 412, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_H, 3, check)
      call createCurrent(P_u, P_g, P_uAnti, P_H, 299, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_H, P_g, 110, check)
      call createCurrent(P_sAnti, P_g, P_s, P_H, P_H, 486, check)
      call createCurrent(P_c, P_H, P_g, P_H, P_cAnti, 169, check)
      call createCurrent(P_c, P_g, P_cAnti, 78, check)
      call createCurrent(P_H, P_cAnti, P_c, P_H, P_g, 350, check)
      call createCurrent(P_g, P_H, P_cAnti, P_H, P_c, 351, check)
      call createCurrent(P_uAnti, P_H, P_H, P_g, P_u, 301, check)
      call createCurrent(P_d, P_H, P_H, P_g, P_dAnti, 344, check)
      call createCurrent(P_H, P_sAnti, P_g, P_s, P_H, 175, check)
      call createCurrent(P_dAnti, P_H, P_d, P_H, P_g, 532, check)
      call createCurrent(P_uAnti, P_u, P_g, 141, check)
      call createCurrent(P_H, P_cAnti, P_c, P_g, 602, check)
      call createCurrent(P_b, P_H, P_bAnti, P_g, 531, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_H, 5, check)
      call createCurrent(P_g, P_b, P_H, P_bAnti, P_H, 408, check)
      call createCurrent(P_g, P_s, P_sAnti, P_H, 439, check)
      call createCurrent(P_g, P_s, P_H, P_H, P_sAnti, 447, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_H, P_g, P_g, 120, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_H, P_g, P_H, 126, check)
      call createCurrent(P_dAnti, P_g, P_d, P_H, P_H, 440, check)
      call createCurrent(P_sAnti, P_H, P_s, 367, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_g, 9, check)
      call createCurrent(P_ghGAnti, P_ghG, P_g, 134, check)
      call createCurrent(P_H, P_H, P_g, P_dAnti, P_d, 235, check)
      call createCurrent(P_H, P_g, P_u, P_H, P_uAnti, 211, check)
      call createCurrent(P_H, P_g, P_g, P_H, 119, check)
      call createCurrent(P_b, P_g, P_H, P_bAnti, 266, check)
      call createCurrent(P_H, P_H, P_uAnti, P_u, P_g, 614, check)
      call createCurrent(P_g, P_H, P_d, P_H, P_dAnti, 323, check)
      call createCurrent(P_H, P_bAnti, P_g, P_H, P_b, 386, check)
      call createCurrent(P_H, P_H, P_g, P_bAnti, P_b, 417, check)
      call createCurrent(P_u, P_H, P_uAnti, P_g, 274, check)
      call createCurrent(P_dAnti, P_g, P_H, P_d, P_H, 290, check)
      call createCurrent(P_g, P_bAnti, P_b, P_H, 306, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_g, P_g, P_H, 54, check)
      call createCurrent(P_sAnti, P_H, P_H, P_g, P_s, 162, check)
      call createCurrent(P_g, P_H, P_u, P_uAnti, 518, check)
      call createCurrent(P_s, P_sAnti, P_H, P_g, P_H, 210, check)
      call createCurrent(P_H, P_H, P_b, P_bAnti, P_g, 500, check)
      call createCurrent(P_H, P_c, P_g, P_H, P_cAnti, 339, check)
      call createCurrent(P_c, P_g, P_H, P_H, P_cAnti, 593, check)
      call createCurrent(P_g, P_g, P_g, P_g, 34, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_g, 121, check)
      call createCurrent(P_d, P_H, P_g, P_H, P_dAnti, 585, check)
      call createCurrent(P_H, P_g, P_cAnti, P_c, P_H, 164, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_H, P_H, 67, check)
      call createCurrent(P_H, P_d, P_g, P_dAnti, 574, check)
      call createCurrent(P_bAnti, P_g, P_b, P_H, P_H, 381, check)
      call createCurrent(P_H, P_uAnti, P_u, P_g, 416, check)
      call createCurrent(P_uAnti, P_H, P_H, P_u, 433, check)
      call createCurrent(P_dAnti, P_H, P_g, P_d, P_H, 530, check)
      call createCurrent(P_H, P_b, P_g, P_bAnti, P_H, 326, check)
      call createCurrent(P_g, P_H, P_c, P_cAnti, 337, check)
      call createCurrent(P_dAnti, P_g, P_H, P_d, 223, check)
      call createCurrent(P_g, P_H, P_cAnti, P_c, 213, check)
      call createCurrent(P_H, P_sAnti, P_H, P_s, P_g, 628, check)
      call createCurrent(P_bAnti, P_b, P_H, P_H, 562, check)
      call createCurrent(P_g, P_H, P_H, P_b, P_bAnti, 479, check)
      call createCurrent(P_uAnti, P_H, P_u, 426, check)
      call createCurrent(P_H, P_g, P_d, P_dAnti, P_H, 517, check)
      call createCurrent(P_H, P_H, P_dAnti, P_d, P_g, 421, check)
      call createCurrent(P_H, P_d, P_dAnti, 436, check)
      call createCurrent(P_H, P_d, P_dAnti, P_g, P_H, 455, check)
      call createCurrent(P_s, P_H, P_sAnti, 584, check)
      call createCurrent(P_H, P_sAnti, P_g, P_s, 330, check)
      call createCurrent(P_g, P_g, P_g, P_g, P_H, P_H, P_H, 102, check)
      call createCurrent(P_H, P_dAnti, P_g, P_d, P_H, 229, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_g, 144, check)
      call createCurrent(P_dAnti, P_H, P_H, P_d, 311, check)
      call createCurrent(P_H, P_d, P_H, P_g, P_dAnti, 480, check)
      call createCurrent(P_u, P_g, P_H, P_uAnti, 403, check)
      call createCurrent(P_H, P_g, P_g, 12, check)
      call createCurrent(P_cAnti, P_H, P_H, P_g, P_c, 560, check)
      call createCurrent(P_H, P_g, P_bAnti, P_H, P_b, 506, check)
      call createCurrent(P_b, P_H, P_g, P_bAnti, P_H, 630, check)
      call createCurrent(P_g, P_b, P_H, P_H, P_bAnti, 551, check)
      call createCurrent(P_g, P_H, P_H, P_cAnti, P_c, 353, check)
      call createCurrent(P_g, P_cAnti, P_H, P_H, P_c, 482, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_H, P_H, 71, check)
      call createCurrent(P_dAnti, P_d, P_g, 53, check)
      call createCurrent(P_H, P_s, P_sAnti, P_g, 182, check)
      call createCurrent(P_H, P_H, P_g, P_H, P_g, P_g, 21, check)
      call createCurrent(P_cAnti, P_c, P_H, P_g, P_H, 360, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_g, P_g, P_H, 82, check)
      call createCurrent(P_b, P_H, P_g, P_H, P_bAnti, 653, check)
      call createCurrent(P_bAnti, P_g, P_b, 37, check)
      call createCurrent(P_c, P_cAnti, 151, check)
      call createCurrent(P_g, P_c, P_cAnti, P_H, 180, check)
      call createCurrent(P_u, P_uAnti, 154, check)
      call createCurrent(P_H, P_H, P_cAnti, P_g, P_c, 247, check)
      call createCurrent(P_d, P_dAnti, P_H, P_H, 552, check)
      call createCurrent(P_H, P_g, P_d, P_dAnti, 340, check)
      call createCurrent(P_sAnti, P_s, P_H, P_H, 621, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_g, P_g, 136, check)
      call createCurrent(P_uAnti, P_g, P_H, P_u, P_H, 369, check)
      call createCurrent(P_dAnti, P_d, P_g, P_H, 499, check)
      call createCurrent(P_s, P_sAnti, P_g, 118, check)
      call createCurrent(P_H, P_c, P_g, P_cAnti, P_H, 478, check)
      call createCurrent(P_g, P_u, P_uAnti, P_H, 245, check)
      call createCurrent(P_H, P_H, P_s, P_g, P_sAnti, 332, check)
      call createCurrent(P_g, P_uAnti, P_u, 74, check)
      call createCurrent(P_dAnti, P_H, P_d, P_g, 505, check)
      call createCurrent(P_H, P_b, P_H, P_bAnti, 536, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_H, P_H, P_g, 22, check)
      call createCurrent(P_g, P_dAnti, P_d, P_H, P_H, 520, check)
      call createCurrent(P_d, P_g, P_H, P_dAnti, P_H, 648, check)
      call createCurrent(P_H, P_uAnti, P_u, P_H, P_g, 409, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_H, P_g, 2, check)
      call createCurrent(P_g, P_H, P_bAnti, P_b, P_H, 491, check)
      call createCurrent(P_g, P_b, P_bAnti, P_H, P_H, 645, check)
      call createCurrent(P_sAnti, P_g, P_s, P_H, 631, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_H, P_g, P_H, 137, check)
      call createCurrent(P_H, P_sAnti, P_g, P_H, P_s, 663, check)
      call createCurrent(P_H, P_H, P_b, P_bAnti, 391, check)
      call createCurrent(P_g, P_H, P_s, P_H, P_sAnti, 368, check)
      call createCurrent(P_g, P_g, P_H, P_g, 31, check)
      call createCurrent(P_c, P_H, P_cAnti, P_H, P_g, 484, check)
      call createCurrent(P_H, P_g, P_H, P_g, 68, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_H, 62, check)
      call createCurrent(P_u, P_uAnti, P_g, P_H, P_H, 598, check)
      call createCurrent(P_bAnti, P_g, P_b, P_H, 282, check)
      call createCurrent(P_g, P_H, P_dAnti, P_H, P_d, 537, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_H, P_g, P_H, 115, check)
      call createCurrent(P_cAnti, P_c, P_H, P_H, 650, check)
      call createCurrent(P_g, P_c, P_H, P_H, P_cAnti, 239, check)
      call createCurrent(P_H, P_b, P_bAnti, P_g, 624, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_H, P_g, 98, check)
      call createCurrent(P_H, P_b, P_H, P_g, P_bAnti, 184, check)
      call createCurrent(P_g, P_cAnti, P_c, P_H, P_H, 443, check)
      call createCurrent(P_H, P_u, P_H, P_uAnti, 207, check)
      call createCurrent(P_g, P_bAnti, P_H, P_H, P_b, 348, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_g, P_H, 100, check)
      call createCurrent(P_cAnti, P_H, P_c, P_g, P_H, 189, check)
      call createCurrent(P_H, P_u, P_uAnti, P_H, P_g, 321, check)
      call createCurrent(P_b, P_H, P_bAnti, P_H, 304, check)
      call createCurrent(P_c, P_H, P_g, P_cAnti, 230, check)
      call createCurrent(P_c, P_cAnti, P_g, P_H, P_H, 303, check)
      call createCurrent(P_H, P_cAnti, P_c, P_H, 514, check)
      call createCurrent(P_H, P_g, P_H, P_H, P_g, P_g, P_g, 143, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_H, 104, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_H, P_H, 58, check)
      call createCurrent(P_d, P_g, P_dAnti, P_H, 293, check)
      call createCurrent(P_bAnti, P_H, P_b, 642, check)
      call createCurrent(P_H, P_sAnti, P_s, P_H, P_g, 468, check)
      call createCurrent(P_H, P_H, P_g, P_u, P_uAnti, 176, check)
      call createCurrent(P_bAnti, P_H, P_b, P_H, 595, check)
      call createCurrent(P_s, P_H, P_g, P_H, P_sAnti, 253, check)
      call createCurrent(P_b, P_H, P_g, P_bAnti, 354, check)
      call createCurrent(P_sAnti, P_H, P_g, P_s, 271, check)
      call createCurrent(P_d, P_H, P_H, P_dAnti, P_g, 167, check)
      call createCurrent(P_g, P_s, P_H, P_sAnti, 192, check)
      call createCurrent(P_dAnti, P_d, P_H, P_H, 276, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_g, P_g, 27, check)
      call createCurrent(P_cAnti, P_H, P_c, 523, check)
      call createCurrent(P_g, P_uAnti, P_H, P_H, P_u, 589, check)
      call createCurrent(P_H, P_bAnti, P_b, P_H, P_g, 270, check)
      call createCurrent(P_d, P_H, P_dAnti, P_H, 613, check)
      call createCurrent(P_H, P_g, P_bAnti, P_b, P_H, 285, check)
      call createCurrent(P_H, P_H, P_bAnti, P_b, 244, check)
      call createCurrent(P_H, P_g, P_H, P_s, P_sAnti, 252, check)
      call createCurrent(P_u, P_H, P_H, P_uAnti, P_g, 224, check)
      call createCurrent(P_bAnti, P_H, P_b, P_H, P_g, 352, check)
      call createCurrent(P_H, P_g, P_sAnti, P_s, 300, check)
      call createCurrent(P_H, P_d, P_dAnti, P_H, P_g, 263, check)
      call createCurrent(P_g, P_g, P_g, P_g, P_H, P_H, 147, check)
      call createCurrent(P_g, P_H, P_H, P_H, P_g, 75, check)
      call createCurrent(P_H, P_dAnti, P_d, P_g, 510, check)
      call createCurrent(P_g, P_uAnti, P_H, P_u, 563, check)
      call createCurrent(P_H, P_H, P_H, P_H, 46, check)
      call createCurrent(P_cAnti, P_c, P_H, P_H, P_g, 312, check)
      call createCurrent(P_s, P_H, P_sAnti, P_H, 296, check)
      call createCurrent(P_H, P_uAnti, P_g, P_u, P_H, 414, check)
      call createCurrent(P_H, P_dAnti, P_g, P_d, 346, check)
      call createCurrent(P_H, P_H, P_g, P_c, P_cAnti, 237, check)
      call createCurrent(P_cAnti, P_H, P_H, P_c, 278, check)
      call createCurrent(P_H, P_H, P_c, P_g, P_cAnti, 181, check)
      call createCurrent(P_g, P_H, P_sAnti, P_s, 199, check)
      call createCurrent(P_g, P_s, P_sAnti, 105, check)
      call createCurrent(P_H, P_c, P_cAnti, P_g, 225, check)
      call createCurrent(P_c, P_cAnti, P_H, P_g, 283, check)
      call createCurrent(P_H, P_bAnti, P_H, P_g, P_b, 626, check)
      call createCurrent(P_uAnti, P_u, P_H, P_H, 392, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_g, P_g, 56, check)
      call createCurrent(P_g, P_b, P_H, P_bAnti, 259, check)
      call createCurrent(P_H, P_g, P_H, P_c, P_cAnti, 205, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_H, 138, check)
      call createCurrent(P_u, P_uAnti, P_g, P_H, 597, check)
      call createCurrent(P_H, P_H, P_b, P_g, P_bAnti, 618, check)
      call createCurrent(P_d, P_dAnti, P_H, P_g, P_H, 637, check)
      call createCurrent(P_d, P_H, P_g, P_dAnti, 581, check)
      call createCurrent(P_dAnti, P_d, P_H, 238, check)
      call createCurrent(P_uAnti, P_H, P_g, P_u, 501, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_g, P_H, P_g, 103, check)
      call createCurrent(P_s, P_sAnti, P_g, P_H, 497, check)
      call createCurrent(P_H, P_g, P_c, P_cAnti, 171, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_H, P_g, P_g, 18, check)
      call createCurrent(P_H, P_u, P_H, P_g, P_uAnti, 438, check)
      call createCurrent(P_g, P_H, P_cAnti, P_c, P_H, 308, check)
      call createCurrent(P_d, P_g, P_dAnti, P_H, P_H, 666, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_g, P_H, 133, check)
      call createCurrent(P_H, P_g, P_cAnti, P_c, 196, check)
      call createCurrent(P_H, P_bAnti, P_g, P_b, 305, check)
      call createCurrent(P_bAnti, P_b, 150, check)
      call createCurrent(P_g, P_g, P_g, 70, check)
      call createCurrent(P_H, P_bAnti, P_b, P_H, 168, check)
      call createCurrent(P_s, P_g, P_sAnti, P_H, 265, check)
      call createCurrent(P_c, P_H, P_g, P_cAnti, P_H, 389, check)
      call createCurrent(P_c, P_cAnti, P_H, P_H, 564, check)
      call createCurrent(P_g, P_H, P_H, P_uAnti, P_u, 441, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_g, P_H, P_g, 91, check)
      call createCurrent(P_d, P_dAnti, P_g, P_H, P_H, 399, check)
      call createCurrent(P_H, P_s, P_g, P_sAnti, P_H, 284, check)
      call createCurrent(P_u, P_uAnti, P_H, P_g, 214, check)
      call createCurrent(P_cAnti, P_g, P_c, P_H, 273, check)
      call createCurrent(P_s, P_H, P_sAnti, P_g, 375, check)
      call createCurrent(P_cAnti, P_g, P_H, P_c, P_H, 177, check)
      call createCurrent(P_sAnti, P_g, P_H, P_H, P_s, 545, check)
      call createCurrent(P_H, P_H, P_sAnti, P_g, P_s, 457, check)
      call createCurrent(P_g, P_cAnti, P_H, P_c, P_H, 342, check)
      call createCurrent(P_cAnti, P_H, P_g, P_c, 576, check)
      call createCurrent(P_d, P_dAnti, P_H, 398, check)
      call createCurrent(P_bAnti, P_b, P_g, P_H, 251, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_g, P_g, P_H, 76, check)
      call createCurrent(P_H, P_d, P_dAnti, P_g, 656, check)
      call createCurrent(P_b, P_bAnti, P_g, 26, check)
      call createCurrent(P_sAnti, P_s, P_g, P_H, P_H, 451, check)
      call createCurrent(P_g, P_H, P_sAnti, P_s, P_H, 610, check)
      call createCurrent(P_H, P_s, P_g, P_sAnti, 240, check)
      call createCurrent(P_u, P_g, P_uAnti, P_H, P_H, 544, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_H, P_H, P_g, 44, check)
      call createCurrent(P_dAnti, P_d, P_H, P_g, 476, check)
      call createCurrent(P_g, P_uAnti, P_u, P_H, P_H, 591, check)
      call createCurrent(P_g, P_c, P_cAnti, 65, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_g, P_H, 45, check)
      call createCurrent(P_c, P_g, P_H, P_cAnti, 607, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_H, P_H, 79, check)
      call createCurrent(P_g, P_sAnti, P_s, P_H, P_H, 166, check)
      call createCurrent(P_H, P_g, P_H, P_cAnti, P_c, 655, check)
      call createCurrent(P_d, P_g, P_dAnti, 142, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_g, P_H, P_H, 14, check)
      call createCurrent(P_sAnti, P_H, P_s, P_H, P_g, 428, check)
      call createCurrent(P_u, P_H, P_H, P_uAnti, 307, check)
      call createCurrent(P_bAnti, P_b, P_H, P_g, 295, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_H, P_g, 6, check)
      call createCurrent(P_g, P_d, P_H, P_dAnti, P_H, 356, check)
      call createCurrent(P_u, P_H, P_H, P_g, P_uAnti, 194, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_H, P_g, P_g, 85, check)
      call createCurrent(P_H, P_s, P_H, P_sAnti, P_g, 494, check)
      call createCurrent(P_dAnti, P_H, P_d, P_H, 587, check)
      call createCurrent(P_u, P_uAnti, P_H, P_H, 664, check)
      call createCurrent(P_dAnti, P_d, P_H, P_g, P_H, 489, check)
      call createCurrent(P_cAnti, P_c, P_g, P_H, 526, check)
      call createCurrent(P_uAnti, P_g, P_H, P_H, P_u, 423, check)
      call createCurrent(P_H, P_uAnti, P_u, P_g, P_H, 611, check)
      call createCurrent(P_g, P_dAnti, P_d, P_H, 345, check)
      call createCurrent(P_g, P_u, P_H, P_H, P_uAnti, 643, check)
      call createCurrent(P_s, P_g, P_H, P_sAnti, 322, check)
      call createCurrent(P_g, P_sAnti, P_s, 55, check)
      call createCurrent(P_H, P_bAnti, P_b, 567, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_H, P_g, 139, check)
      call createCurrent(P_g, P_u, P_uAnti, P_H, P_H, 272, check)
      call createCurrent(P_sAnti, P_H, P_s, P_H, 427, check)
      call createCurrent(P_c, P_H, P_H, P_cAnti, 511, check)
      call createCurrent(P_H, P_g, P_bAnti, P_b, 578, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_H, 20, check)
      call createCurrent(P_H, P_g, P_d, P_H, P_dAnti, 449, check)
      call createCurrent(P_g, P_g, P_H, P_H, P_g, P_H, P_g, 0, check)
      call createCurrent(P_u, P_H, P_uAnti, 393, check)
      call createCurrent(P_g, P_b, P_bAnti, 61, check)
      call createCurrent(P_g, P_g, P_H, P_H, 80, check)
      call createCurrent(P_s, P_g, P_sAnti, P_H, P_H, 159, check)
      call createCurrent(P_uAnti, P_g, P_H, P_u, 623, check)
      call createCurrent(P_c, P_cAnti, P_g, P_H, 343, check)
      call createCurrent(P_g, P_dAnti, P_H, P_H, P_d, 219, check)
      call createCurrent(P_u, P_g, P_uAnti, 135, check)
      call createCurrent(P_uAnti, P_H, P_g, P_H, P_u, 546, check)
      call createCurrent(P_g, P_c, P_H, P_cAnti, P_H, 336, check)
      call createCurrent(P_c, P_H, P_cAnti, 264, check)
      call createCurrent(P_s, P_sAnti, 149, check)
      call createCurrent(P_cAnti, P_c, 155, check)
      call createCurrent(P_uAnti, P_g, P_u, P_H, 216, check)
      call createCurrent(P_sAnti, P_s, P_g, 113, check)
      call createCurrent(P_H, P_g, P_u, P_uAnti, P_H, 218, check)
      call createCurrent(P_g, P_H, P_H, P_s, P_sAnti, 640, check)
      call createCurrent(P_H, P_g, P_sAnti, P_H, P_s, 404, check)
      call createCurrent(P_H, P_bAnti, P_H, P_b, 212, check)
      call createCurrent(P_s, P_sAnti, P_H, P_H, 651, check)
      call createCurrent(P_b, P_bAnti, P_H, P_g, P_H, 193, check)
      call createCurrent(P_dAnti, P_g, P_d, P_H, 419, check)
      call createCurrent(P_u, P_g, P_H, P_H, P_uAnti, 161, check)
      call createCurrent(P_g, P_g, P_g, P_H, 84, check)
      call createCurrent(P_H, P_H, P_H, P_g, P_g, P_g, P_g, 32, check)
      call createCurrent(P_dAnti, P_H, P_g, P_d, 424, check)
      call createCurrent(P_H, P_H, P_cAnti, P_c, P_g, 366, check)
      call createCurrent(P_bAnti, P_H, P_b, P_g, 522, check)
      call createCurrent(P_H, P_H, P_g, P_H, P_g, P_g, P_g, 112, check)
      call createCurrent(P_b, P_g, P_bAnti, 131, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_g, P_H, P_g, 15, check)
      call createCurrent(P_H, P_sAnti, P_s, P_g, 379, check)
      call createCurrent(P_d, P_H, P_dAnti, 362, check)
      call createCurrent(P_H, P_H, P_u, P_uAnti, 365, check)
      call createCurrent(P_c, P_H, P_cAnti, P_H, 488, check)
      call createCurrent(P_uAnti, P_g, P_u, 107, check)
      call createCurrent(P_g, P_H, P_s, P_sAnti, 657, check)
      call createCurrent(P_g, P_H, P_bAnti, P_b, 437, check)
      call createCurrent(P_H, P_H, P_c, P_cAnti, 586, check)
      call createCurrent(P_uAnti, P_H, P_g, P_u, P_H, 250, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_g, P_g, P_H, 10, check)
      call createCurrent(P_bAnti, P_H, P_g, P_b, P_H, 620, check)
      call createCurrent(P_s, P_sAnti, P_g, P_H, P_H, 565, check)
      call createCurrent(P_H, P_g, P_H, P_H, P_g, P_g, 60, check)
      call createCurrent(P_g, P_H, P_c, P_cAnti, P_H, 525, check)
      call createCurrent(P_H, P_H, P_g, P_d, P_dAnti, 333, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_g, 111, check)
      call createCurrent(P_g, P_bAnti, P_H, P_b, 649, check)
      call createCurrent(P_c, P_cAnti, P_g, 43, check)
      call createCurrent(P_H, P_uAnti, P_g, P_u, 286, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_g, 23, check)
      call createCurrent(P_H, P_g, P_H, P_d, P_dAnti, 243, check)
      call createCurrent(P_H, P_u, P_H, P_uAnti, P_g, 608, check)
      call createCurrent(P_H, P_g, P_g, P_g, P_H, P_g, 90, check)
      call createCurrent(P_H, P_g, P_H, P_dAnti, P_d, 382, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_g, P_H, P_g, 94, check)
      call createCurrent(P_H, P_sAnti, P_H, P_g, P_s, 503, check)
      call createCurrent(P_b, P_bAnti, P_H, P_H, 647, check)
      call createCurrent(P_H, P_H, P_s, P_sAnti, 572, check)
      call createCurrent(P_s, P_H, P_sAnti, P_g, P_H, 380, check)
      call createCurrent(P_b, P_g, P_bAnti, P_H, 302, check)
      call createCurrent(P_g, P_H, P_H, P_sAnti, P_s, 464, check)
      call createCurrent(P_g, P_sAnti, P_H, P_H, P_s, 609, check)
      call createCurrent(P_H, P_dAnti, P_d, 335, check)
      call createCurrent(P_H, P_H, P_u, P_g, P_uAnti, 625, check)
      call createCurrent(P_g, P_bAnti, P_b, P_H, P_H, 442, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_g, P_H, 40, check)
      call createCurrent(P_g, P_sAnti, P_H, P_s, P_H, 246, check)
      call createCurrent(P_uAnti, P_H, P_u, P_H, 453, check)
      call createCurrent(P_H, P_H, P_dAnti, P_g, P_d, 588, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_g, P_H, 114, check)
      call createCurrent(P_g, P_H, P_H, P_u, P_uAnti, 622, check)
      call createCurrent(P_H, P_g, P_H, P_g, P_g, P_g, 33, check)
      call createCurrent(P_ghG, P_g, P_ghGAnti, 124, check)
      call createCurrent(P_H, P_cAnti, P_H, P_g, P_c, 431, check)
      call createCurrent(P_uAnti, P_H, P_H, P_u, P_g, 402, check)
      call createCurrent(P_H, P_s, P_sAnti, 550, check)
      call createCurrent(P_g, P_H, P_sAnti, P_H, P_s, 200, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_g, 63, check)
      call createCurrent(P_sAnti, P_H, P_g, P_s, P_H, 665, check)
      call createCurrent(P_H, P_c, P_cAnti, 228, check)
      call createCurrent(P_bAnti, P_H, P_H, P_b, 592, check)
      call createCurrent(P_H, P_g, P_s, P_sAnti, P_H, 268, check)
      call createCurrent(P_bAnti, P_b, P_H, P_H, P_g, 173, check)
      call createCurrent(P_g, P_dAnti, P_d, 129, check)
      call createCurrent(P_g, P_H, P_uAnti, P_u, 355, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_H, P_g, 128, check)
      call createCurrent(P_g, P_g, P_g, P_H, P_g, P_H, 39, check)
      call createCurrent(P_bAnti, P_b, P_H, 242, check)
      call createCurrent(P_H, P_g, P_uAnti, P_u, 188, check)
      call createCurrent(P_H, P_b, P_H, P_bAnti, P_g, 316, check)
      call createCurrent(P_g, P_bAnti, P_b, 66, check)
      call createCurrent(P_H, P_H, P_H, P_g, P_g, 81, check)
      call createCurrent(P_H, P_H, P_g, P_g, P_g, P_H, 19, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_g, P_H, P_g, 57, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_g, P_g, 48, check)
      call createCurrent(P_s, P_H, P_g, P_sAnti, 542, check)
      call createCurrent(P_H, P_u, P_uAnti, P_H, 554, check)
      call createCurrent(P_H, P_g, P_b, P_bAnti, 359, check)
      call createCurrent(P_uAnti, P_H, P_u, P_H, P_g, 519, check)
      call createCurrent(P_uAnti, P_u, P_H, 605, check)
      call createCurrent(P_cAnti, P_g, P_H, P_H, P_c, 504, check)
      call createCurrent(P_b, P_bAnti, P_H, P_g, 446, check)
      call createCurrent(P_g, P_H, P_b, P_bAnti, 490, check)
      call createCurrent(P_cAnti, P_c, P_g, P_H, P_H, 371, check)
      call createCurrent(P_H, P_s, P_H, P_sAnti, 325, check)
      call createCurrent(P_H, P_H, P_d, P_dAnti, P_g, 215, check)
      call createCurrent(P_H, P_sAnti, P_s, P_g, P_H, 659, check)
      call createCurrent(P_cAnti, P_H, P_g, P_H, P_c, 317, check)
      call createCurrent(P_sAnti, P_s, P_H, P_g, 160, check)
      call createCurrent(P_H, P_bAnti, P_g, P_b, P_H, 667, check)
      call createCurrent(P_c, P_g, P_H, P_cAnti, P_H, 241, check)
      call createCurrent(P_H, P_uAnti, P_H, P_u, P_g, 549, check)
      call createCurrent(P_sAnti, P_H, P_H, P_s, 204, check)
      call createCurrent(P_g, P_sAnti, P_s, P_H, 509, check)
      call createCurrent(P_sAnti, P_s, P_g, P_H, 190, check)
      call createCurrent(P_sAnti, P_H, P_s, P_g, 267, check)
      call createCurrent(P_H, P_bAnti, P_b, P_g, 558, check)
      call createCurrent(P_g, P_c, P_H, P_cAnti, 470, check)
      call createCurrent(P_g, P_d, P_H, P_H, P_dAnti, 473, check)
      call createCurrent(P_H, P_u, P_uAnti, 636, check)
      call createCurrent(P_s, P_H, P_H, P_sAnti, 358, check)
      call createCurrent(P_g, P_H, P_H, P_d, P_dAnti, 616, check)
      call createCurrent(P_cAnti, P_H, P_c, P_H, P_g, 387, check)
      call createCurrent(P_H, P_H, P_bAnti, P_g, P_b, 604, check)
      call createCurrent(P_b, P_bAnti, P_g, P_H, 331, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_H, 97, check)
      call createCurrent(P_g, P_H, P_H, P_dAnti, P_d, 287, check)
      call createCurrent(P_g, P_ghG, P_ghGAnti, 130, check)
      call createCurrent(P_bAnti, P_H, P_b, P_g, P_H, 471, check)
      call createCurrent(P_g, P_H, P_g, P_H, 109, check)
      call createCurrent(P_H, P_b, P_bAnti, 383, check)
      call createCurrent(P_g, P_H, P_g, P_g, 4, check)
      call createCurrent(P_cAnti, P_H, P_c, P_H, 364, check)
      call createCurrent(P_bAnti, P_g, P_H, P_H, P_b, 279, check)
      call createCurrent(P_g, P_H, P_H, P_g, P_g, 13, check)
      call createCurrent(P_bAnti, P_g, P_H, P_b, 400, check)
      call createCurrent(P_g, P_ghGAnti, P_ghG, 127, check)
      call createCurrent(P_g, P_g, 156, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_H, P_H, P_g, 108, check)
      call createCurrent(P_H, P_cAnti, P_g, P_c, P_H, 606, check)
      call createCurrent(P_bAnti, P_g, P_H, P_b, P_H, 617, check)
      call createCurrent(P_H, P_s, P_H, P_g, P_sAnti, 418, check)
      call createCurrent(P_cAnti, P_c, P_H, 452, check)
      call createCurrent(P_H, P_b, P_bAnti, P_g, P_H, 635, check)
      call createCurrent(P_g, P_H, P_uAnti, P_u, P_H, 361, check)
      call createCurrent(P_u, P_H, P_g, P_uAnti, P_H, 363, check)
      call createCurrent(P_g, P_cAnti, P_H, P_c, 291, check)
      call createCurrent(P_cAnti, P_c, P_H, P_g, 600, check)
      call createCurrent(P_H, P_uAnti, P_u, P_H, 660, check)
      call createCurrent(P_b, P_bAnti, 157, check)
      call createCurrent(P_s, P_H, P_H, P_g, P_sAnti, 401, check)
      call createCurrent(P_g, P_g, P_H, P_g, P_g, P_H, P_H, 16, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_g, 25, check)
      call createCurrent(P_u, P_uAnti, P_g, 83, check)
      call createCurrent(P_g, P_H, P_c, P_H, P_cAnti, 208, check)
      call createCurrent(P_g, P_s, P_H, P_sAnti, P_H, 644, check)
      call createCurrent(P_H, P_uAnti, P_g, P_H, P_u, 288, check)
      call createCurrent(P_H, P_bAnti, P_H, P_b, P_g, 524, check)
      call createCurrent(P_H, P_cAnti, P_g, P_H, P_c, 543, check)
      call createCurrent(P_g, P_dAnti, P_H, P_d, 292, check)
      call createCurrent(P_H, P_u, P_g, P_uAnti, P_H, 553, check)
      call createCurrent(P_g, P_H, P_g, P_H, P_g, P_g, 30, check)
      call createCurrent(P_u, P_H, P_uAnti, P_H, 280, check)
      call createCurrent(P_b, P_H, P_bAnti, P_g, P_H, 483, check)
      call createCurrent(P_bAnti, P_b, P_H, P_g, P_H, 465, check)
      call createCurrent(P_H, P_H, P_sAnti, P_s, P_g, 281, check)
      call createCurrent(P_H, P_dAnti, P_d, P_H, 448, check)
      call createCurrent(P_H, P_b, P_bAnti, P_H, P_g, 559, check)
      call createCurrent(P_g, P_uAnti, P_u, P_H, 569, check)
      call createCurrent(P_g, P_H, P_g, P_g, P_g, P_H, P_H, 145, check)
      call createCurrent(P_dAnti, P_d, P_H, P_H, P_g, 396, check)
      call createCurrent(P_g, P_H, P_H, P_H, P_g, P_g, P_g, 28, check)
      call createCurrent(P_cAnti, P_g, P_H, P_c, 202, check)
      call createCurrent(P_H, P_H, P_g, P_H, P_g, 86, check)
      call createCurrent(P_b, P_H, P_bAnti, P_H, P_g, 209, check)
      call createCurrent(P_H, P_g, P_u, P_uAnti, 234, check)
      call createCurrent(P_sAnti, P_g, P_H, P_s, P_H, 658, check)
      call createCurrent(P_g, P_u, P_H, P_uAnti, 261, check)
      call createCurrent(P_s, P_g, P_H, P_sAnti, P_H, 425, check)
      call createCurrent(P_s, P_sAnti, P_H, P_H, P_g, 646, check)
      call createCurrent(P_sAnti, P_H, P_s, P_g, P_H, 420, check)
      call createCurrent(P_H, P_dAnti, P_H, P_d, P_g, 596, check)
      call createCurrent(P_u, P_uAnti, P_H, P_H, P_g, 466, check)
      call createCurrent(P_b, P_H, P_bAnti, 165, check)
      call createCurrent(P_sAnti, P_g, P_H, P_s, 467, check)
      call createCurrent(P_bAnti, P_H, P_H, P_b, P_g, 594, check)
      call createCurrent(P_g, P_c, P_cAnti, P_H, P_H, 577, check)
      call createCurrent(P_d, P_dAnti, P_g, P_H, 231, check)
      call createCurrent(P_H, P_H, P_g, P_s, P_sAnti, 297, check)
      call createCurrent(P_H, P_H, P_g, P_b, P_bAnti, 496, check)
      call createCurrent(P_uAnti, P_u, P_H, P_g, 313, check)
      call createCurrent(P_H, P_bAnti, P_b, P_g, P_H, 654, check)
      call createCurrent(P_H, P_c, P_cAnti, P_H, 633, check)
      call createCurrent(P_H, P_g, P_g, P_H, P_g, P_g, P_H, 89, check)
      call createCurrent(P_bAnti, P_b, P_g, P_H, P_H, 201, check)
    end subroutine createCurrents

    subroutine check_vertices_mdl()
      call createCurrents(.true.)
    end subroutine check_vertices_mdl

    subroutine get_vertex_lorentz_rank_mdl(ty,cc,mr)
      integer, intent(in) :: ty
      logical, intent(in) :: cc(0:)
      integer, intent(out) :: mr
      select case(ty)
      case(17)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(30)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(35)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(39)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(50)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(60)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(74)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(91)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(99)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(113)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(122)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(128)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(132)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(143)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(147)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(148)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(156)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(169)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(173)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(176)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(177)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(178)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(183)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(185)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(190)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(216)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(219)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(223)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(225)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(235)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(243)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(252)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(254)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(256)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(260)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(262)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(267)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(276)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(286)
        if (cc(0) .or. cc(1)) then
          mr = 1
        else
          mr = 0
        end if 
      case(288)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(319)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(325)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(326)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(327)
        if (cc(0) .or. cc(1)) then
          mr = 2
        else
          mr = 0
        end if 
      case(332)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case(337)
        if (cc(0)) then
          mr = 1
        else
          mr = 0
        end if 
      case default
        mr = 0
      end select 
    end subroutine get_vertex_lorentz_rank_mdl
end module class_vertices
