!******************************************************************************!
!                                                                              !
!    lcVSSSV.f90                                                               !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcVSSSV.f90                                    |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 03/2019                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2019-03-21 23:40             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcVSSSV
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35, x36, x37,  &
                  x38, x39, x40, x41, x42, x43, x44, x45, x46, x47
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec, x4vec, x5vec
  contains
    subroutine loopVSSSV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4 &
    , PL4, m, wp, riMaxIn, riMaxOut, rankInc)
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
      complex (kind = dp), intent(in), dimension(0:3) :: w3
      complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
      complex (kind = dp), intent(in), dimension(0:3) :: w4
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(VSSSV0)
!       P(1,5)*P(5,1)*c0
!       Metric(1,5)*P(-1,1)*P(-1,5)*c1
        x1vec = P1(:)*co(0)
        x1 = cima*w2(0)*w3(0)*w4(0)
        do j = riMaxIn, 0, -1
          x0vec = co(1)*wl(:,j)
          x0 = -P1(0)*wl(0,j) + P1(1)*wl(1,j) + P1(2)*wl(2,j) + P1(3)*wl(3,j   &
               ) - P2(0)*wl(0,j) + P2(1)*wl(1,j) + P2(2)*wl(2,j) + P2(3)*wl(3,j&
               ) - P3(0)*wl(0,j) + P3(1)*wl(1,j) + P3(2)*wl(2,j) + P3(3)*wl(3,j&
               ) - P4(0)*wl(0,j) + P4(1)*wl(1,j) + P4(2)*wl(2,j) + P4(3)*wl(3,j&
               )
          x2vec = P1(0)*co(1)*wl(:,j)
          x3vec = P1(1)*co(1)*wl(:,j)
          x4vec = P1(2)*co(1)*wl(:,j)
          x5vec = P1(3)*co(1)*wl(:,j)
          wp(:,j) = -x0*x1*x1vec + x0vec*(P1(0)**2*x1 - P1(1)**2*x1 - P1(2)**2*&
                    x1 - P1(3)**2*x1) + x2vec*(P2(0)*x1 + P3(0)*x1 + P4(0)*x1  &
                    ) + x3vec*(-P2(1)*x1 - P3(1)*x1 - P4(1)*x1) + x4vec*(-P2(2 &
                    )*x1 - P3(2)*x1 - P4(2)*x1) + x5vec*(-P2(3)*x1 - P3(3)*x1  &
                     - P4(3)*x1)
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) - co(0)*gg(:,mu)*x0*x1 + wl(mu,j)*x1*    &
                          x1vec + x0vec*(2*P1(mu)*x1 + P2(mu)*x1 + P3(mu)*x1 + &
                          P4(mu)*x1)
            if(rankInc .lt. 2) cycle
            do nu = 0, 3
              riOut = incRI(nu, incRI(mu, j))
              wp(:,riOut) = wp(:,riOut) + co(0)*gg(:,nu)*wl(mu,j)*x1 + gg(nu,mu&
                            )*x0vec*x1
            end do 
          end do 
        end do 
      end select 
    end subroutine loopVSSSV
end module lcVSSSV
