!******************************************************************************!
!                                                                              !
!    lcVSVSSV.f90                                                              !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcVSVSSV.f90                                   |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 03/2019                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2019-03-21 23:40             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcVSVSSV
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35, x36, x37,  &
                  x38, x39, x40, x41, x42, x43, x44, x45, x46, x47
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec, x4vec, x5vec
  contains
    subroutine loopVSVSSV(ty, co, wl, P1, PL1, w2, P2, PL2, w3, P3, PL3, w4, P4&
    , PL4, w5, P5, PL5, m, wp, riMaxIn, riMaxOut, rankInc)
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (kind = dp), intent(in), dimension(0:3) :: P3, PL3
      complex (kind = dp), intent(in), dimension(0:3) :: w3
      complex (kind = dp), intent(in), dimension(0:3) :: P4, PL4
      complex (kind = dp), intent(in), dimension(0:3) :: w4
      complex (kind = dp), intent(in), dimension(0:3) :: P5, PL5
      complex (kind = dp), intent(in), dimension(0:3) :: w5
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(VSVSSV0)
!       c0*(Metric(1,3)*P(6,1) - Metric(1,3)*P(6,3) - Metric(1,6)*P(3,1) + Metric
!       (1,6)*P(3,6) + Metric(3,6)*P(1,3) - Metric(3,6)*P(1,6))
        x1 = cima*co(0)*w2(0)*w4(0)*w5(0)
        do j = riMaxIn, 0, -1
          x0 = -w3(0)*wl(0,j) + w3(1)*wl(1,j) + w3(2)*wl(2,j) + w3(3)*wl(3,j)
          x0vec = w3(1)*wl(:,j)
          x1vec = w3(2)*wl(:,j)
          x2vec = w3(3)*wl(:,j)
          x3vec = w3(0)*wl(:,j)
          wp(:,j) = -P1(:)*x0*x1 + P3(:)*x0*x1 + w3(:)*(P1(0)*wl(0,j)*x1 - P1(1&
                    )*wl(1,j)*x1 - P1(2)*wl(2,j)*x1 - P1(3)*wl(3,j)*x1 + P2(0)*&
                    wl(0,j)*x1 - P2(1)*wl(1,j)*x1 - P2(2)*wl(2,j)*x1 - P2(3)*wl&
                    (3,j)*x1 + 2*P3(0)*wl(0,j)*x1 - 2*P3(1)*wl(1,j)*x1 - 2*P3(2&
                    )*wl(2,j)*x1 - 2*P3(3)*wl(3,j)*x1 + P4(0)*wl(0,j)*x1 - P4(1&
                    )*wl(1,j)*x1 - P4(2)*wl(2,j)*x1 - P4(3)*wl(3,j)*x1 + P5(0)*&
                    wl(0,j)*x1 - P5(1)*wl(1,j)*x1 - P5(2)*wl(2,j)*x1 - P5(3)*wl&
                    (3,j)*x1) + x0vec*(2*P1(1)*x1 + P2(1)*x1 + P3(1)*x1 + P4(1 &
                    )*x1 + P5(1)*x1) + x1vec*(2*P1(2)*x1 + P2(2)*x1 + P3(2)*x1 &
                     + P4(2)*x1 + P5(2)*x1) + x2vec*(2*P1(3)*x1 + P2(3)*x1 + P3&
                    (3)*x1 + P4(3)*x1 + P5(3)*x1) + x3vec*(-2*P1(0)*x1 - P2(0)*&
                    x1 - P3(0)*x1 - P4(0)*x1 - P5(0)*x1)
          if(rankInc .lt. 1) cycle
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) - gg(:,mu)*x0*x1 + w3(:)*wl(mu,j)*x1 - 2*&
                          w3(mu)*wl(:,j)*x1
          end do 
        end do 
      end select 
    end subroutine loopVSVSSV
end module lcVSVSSV
